<?php
// 電子工作方眼紙生成
// タクトスイッチゲージ
// たけぼうエンジニアリング - https://takebo.wpxblog.jp/

// ページサイズ A4 (1/72インチ単位)
$pgw = floor(210 * 7200 / 25.4) / 100;
$pgh = floor(297 * 7200 / 25.4) / 100;
$spcw = floor(15 * 7200 / 25.4) / 100;
$spch = floor(20 * 7200 / 25.4) / 100;

// 罫線領域
$gu = 0.1 * 72; // 間隔 0.1 inch (2.54 mm)
$gnx = (int)(($pgw - $spcw * 2) / $gu); // 垂直罫線数
$gny = (int)(($pgh - $spch * 2) / $gu); // 水平罫線数
$gx0 = ($pgw - $gu * $gnx) / 2; // 原点X
$gy0 = ($pgh - $gu * $gny) / 2; // 原点Y
$gx1 = $gx0 + $gu * $gnx;
$gy1 = $gy0 + $gu * $gny;
$gw1 = 0.2;  // 細
$gw2 = 0.35; // 中
$gw3 = 0.5;  // 太
$gwd = $gw2; // ドット
$gc = 0.4; // 罫線の濃さ


$draw = [];
$draw[] = "/DeviceGray CS";

// 外枠
$draw[] = "${gw3} w";
$sw = $gw3;
$draw[] = "0.0 SC";
{
	$w = $gu * $gnx;
	$h = $gu * $gny;
	$draw[] = "${gx0} ${gy0} ${w} ${h} re S";
}
// テキスト
{
	$x = $gx0 + 1;
	$y = $gy1 + 2;
	$draw[] = "/FONT1 8 Tf";
	$draw[] = "BT ${x} ${y} Td (6.5 x 4.5 mm Tact Switch Template) Tj ET";
}

$draw[] = "${gc} SC";

// 1/20 inch グリッド
$d0 = $gwd;
$d1 = $gu * .5 - $d0;
$dp = ($gu + $d0) / 2;
$draw[] = "${d0} w";
$draw[] = "[ ${d0} ${d1} ] ${dp} d";
for ($gy = .5; $gy < $gny; $gy += .5) {
	$y = $gy0 + ($gny - $gy) * $gu;
	$draw[] = "${gx0} ${y} m ${gx1} ${y} l";
}
$draw[] = "S d";

$draw[] = "0.0 SC";

// タクトスイッチ配置
$ll = 0.7 * 72 / 25.5; // リード長さ
foreach (['2.5', '3.0', '3.5', '4.0'] as $py) {
	foreach (['3.5', '4.0', '4.5', '5.0'] as $px) {
		$draw2 = $draw;
		// テキスト
		$x = $gx1 - 100;
		$y = $gy1 + 2;
		$draw2[] = "/FONT1 8 Tf";
		$draw2[] = "BT ${x} ${y} Td (pitch X:${px} Y:${py} @2.54 mm) Tj ET";


		// 水平線パラメータ
		$d1 = $ll * 2 + 6.5 * 72 / 25.4; // 描画部分長さ
		$d0 = $px * $gu - $d1; // 間隔
		$dotx = "[ ${d1} ${d0} ] 0 d"; ; // 水平破線
		$nx = (int)(($gu * ($gnx - 3) + $d0) / ($d1 + $d0)); // 水平個数
		$ofsx = -$d1 * .5; // スイッチ中心基準の描画開始位置
		$lx = ($d1 + $d0) * $nx - $d0; // 描画長さ

		// 垂直線パラメータ
		$d1 = $ll * 2 + 4.5 * 72 / 25.4; // 描画部分長さ
		$d0 = $py * $gu - $d1; // 間隔
		$doty = "[ ${d1} ${d0} ] 0 d"; // 垂直破線
		$ny = (int)(($gu * ($gny - 3) + $d0) / ($d1 + $d0)); // 水平個数
		$ofsy = -$d1 * .5; // スイッチ中心基準の描画開始位置
		$ly = ($d1 + $d0) * $ny - $d0; // 描画長さ

		// 水平線
		$draw2[] = $dotx;
		for ($i = 0; $i <= 1; $i++) {
			$gw = $i ? $gw1 : $gw2;
			$draw2[] = "${gw} w";
			$gx = $gx0 + 3 * $gu;
			for ($y = 0; $y < $ny; $y++) {
				$gy = $gy0 + ($gny - ($y * $py + 3)) * $gu;
				$x1 = $gx + $ofsx;
				$x2 = $x1 + $lx;
				if ($i) {
					// 中心線
					$draw2[] = "${x1} ${gy} m ${x2} ${gy} l";
				} else {
					// 外形線
					$y1 = $gy + $ofsy + $ll;
					$draw2[] = "${x1} ${y1} m ${x2} ${y1} l";
					$y1 = $gy - $ofsy - $ll;
					$draw2[] = "${x1} ${y1} m ${x2} ${y1} l";
				}
			}
			$draw2[] = "S";
		}

		// 垂直線
		$draw2[] = $doty;
		for ($i = 0; $i <= 1; $i++) {
			$gw = $i ? $gw1 : $gw2;
			$draw2[] = "${gw} w";
			$gy = $gy0 + ($gny - 3) * $gu;
			for ($x = 0; $x < $nx; $x++) {
				$gx = $gx0 + ($x * $px + 3) * $gu;
				$y1 = $gy - $ofsy;
				$y2 = $y1 - $ly;
				if ($i) {
					// 中心線
					$draw2[] = "${gx} ${y1} m ${gx} ${y2} l";
				} else {
					// 外形線
					$x1 = $gx + $ofsx + $ll;
					$draw2[] = "${x1} ${y1} m ${x1} ${y2} l";
					$x1 = $gx - $ofsx - $ll;
					$draw2[] = "${x1} ${y1} m ${x1} ${y2} l";
				}
			}
			$draw2[] = "S";
		}

		createPaper("TactSWTemplate-${px}-${py}.pdf", $pgw, $pgh, $draw2);
	}
}



// PDF生成
function createPaper($fn, $pgw, $pgh, $draw)
{
	$pdf = [];
	$pdf[0] = '%PDF-1.2';

	// カタログ辞書
	$pdf[1] = '/Type /Catalog /Pages 2 0 R';

	// ページツリーノード
	$pdf[2] = "/Type /Pages /Kids [ 3 0 R ] /MediaBox [ 0 0 ${pgw} ${pgh} ] /Count 1";

	// ページオブジェクト
	$pdf[3] = '/Type /Page /Parent 2 0 R /Resources 4 0 R /Contents 6 0 R';

	// リソース
	$pdf[4] = '/Font << /FONT1 5 0 R >>';

	// フォント
	$pdf[5] = '/Type /Font /Subtype /Type1 /BaseFont /Helvetica';

	// コンテンツストリーム
	$pdf[6] = ['/Filter /FlateDecode', gzcompress(join(' ', $draw))];

	outPdf($fn, $pdf);
}


// PDF出力
function outPdf($fn, $pdf)
{
	$p = 0;
	$xref = [];
	$fp = fopen($fn, 'w');

	// header/body
	foreach ($pdf as $i => $obj) {
		if ($i == 0) {
			// header
			$buf = "$obj\n";
			$xref[$i] = "0000000000 65535 f \n";
		} else {
			// body
			if (is_array($obj)) {
				// with stream
				$size = strlen($obj[1]);
				$buf = "$i 0 obj\n" .
					"<< $obj[0] /Length $size >>\n" .
					"stream\n".
					"$obj[1]\n".
					"endstream\n".
					"endobj\n";
			} else {
				// no stream
				$buf = "$i 0 obj\n" .
					"<< $obj >>\n" .
					"endobj\n";
			}
			$xref[$i] = sprintf("%010d %05d n \n", $p, 0);
		}
		fwrite($fp, $buf);
		$p += strlen($buf);
	}

	// xref
	{
		$ids = array_keys($xref);
		sort($ids);

		$secs = [];
		$s = -1;
		$chk = -1;
		foreach ($ids as $i) {
			if ($i != $chk) {
				$chk = $s = $i;
				$secs[$s] = [];
			}
			$secs[$s][] = $xref[$i];
			$chk++;
		}
		$buf = "xref\n";
		foreach ($secs as $s => $sec) {
			$cnt = count($sec);
			$buf .= "$s $cnt\n" . join('', $sec);
		}
		fwrite($fp, $buf);
	}

	// trailer
	{
		$size = count($pdf);
		$buf = "trailer\n" .
			"<< /Size ${size} /Root 1 0 R >>\n" .
			"startxref\n" .
			"${p}\n" .
			"%%EOF\n";
		fwrite($fp, $buf);
	}

	fclose($fp);
}
